@@ -21,12 +21,10 @@ require Exporter;
 	     emitContextProperties
 	     emitEvalResultAsProperty
 	     emitEvaluatedPropertyGetInfo
-	     emitFinalPropertyValue
 	     figureEncoding
 	     getContextProperties
 	     getFullPropertyInfoByValue
 	     getPropertyInfo
-	     getPropertyValue
 	     sortArrayOfNames
 	     GlobalVars
 	     LocalVars
@@ -67,8 +65,9 @@ $ldebug = 0;
 		      Special => PunctuationVariables);
 @contextProperties = sort { $contextProperties{$a} <=> $contextProperties{$b} } keys %contextProperties;
 
-@_punctuationVariables = ('$_', '$?', '$@', '$.', '@+', '@-', '$+', '$&', '$!', '$\'', '$`', '$$', '$0');
-
+@_punctuationVariables = ('$_', '$?', '$@', '$.', '@+', '@-', '$+', '$!', '$$', '$0');
+# $`, $& and $' are special-cased to avoid the performance penalty
+@_rxPunctuationVariables = ('`', '&', '\'');
 
 # Exported subs
 
@@ -174,21 +173,25 @@ sub emitEvaluatedPropertyGetInfo($$$$$$$) {
 	$pageIndex) = @_;
 
     my $res = sprintf(qq(%s\n<response %s command="%s" 
-			 transaction_id="%s" >),
+			 transaction_id="%s" ),
 		      xmlHeader(),
 		      namespaceAttr(),
 		      $cmd,
 		      $transactionID);
     my $finalVal = $nameAndValue->[NV_VALUE];
-    
+
+    $res .= '>' if $cmd ne 'property_value';
+    my $startTag = $cmd ne 'property_value' ? '<property' : '';
+    my $endTag = $cmd ne 'property_value' ? '</property>' : '';
     my $finalName = makeFullName($property_long_name, $propertyKey);
-    $res .= getFullPropertyInfoByValue($propertyKey || $finalName, # name
-				       $finalName,
-				       $finalVal,
-				       $maxDataSize,
-				       $pageIndex, # page
-				       0, # current depth
-				       );
+    $res .= _getFullPropertyInfoByValue($startTag, $endTag,
+					$propertyKey || $finalName, # name
+					$finalName,
+					$finalVal,
+					$maxDataSize,
+					$pageIndex, # page
+					0, # current depth
+					);
     $res .= "\n</response>";
     printWithLength($res);
 }
@@ -201,68 +204,30 @@ sub emitEvaluatedPropertyGetInfo($$$$$$$) {
 
 sub getContextProperties($$) {
     my ($context_id, $packageName) = @_;
-    
+
     # Here just show the top-level.
-    my @results;
     local $settings{max_depth}[0] = 0;
     if ($context_id == GlobalVars) {
+	require B if $] >= 5.010;
 	# Globals
 	# Variables on the calling frame
-	# To get the vars, we need the '::' at the end
-	$packageName =~ s/(?<!::)$/::/;
+	my $stash = \%{"${packageName}::"};
 	my @results;
-	eval {
-	    my @data;
-	    open my $ah, '>', \my $data;
-	    defined &main::dumpValue || do 'dumpvar.pl';
-	    if (defined &main::dumpValue) {
-		my $oh = select $ah;
-		# must detect sigpipe failures  - not catching
-		# then will cause the debugger to die.
-		eval {
-		    &main::dumpvar($packageName, -1);
-		};
-
-		# The die does not need to include the $@, because 
-		# it will automatically get propagated for us.
-		if ($@) {
-		    dblog("eval globals: $@\n");
-		} else {
-		    my @hits = ($data =~ m/^([\$\@\%][\w_]+)\s+=\s+/gm);
-		    foreach my $h (@hits) {
-			# dblog("examining '$h' in package [$packageName]\n") if $ldebug;
-			my ($h1, $h2) = split(//, $h, 2);
-			if ($h1 ne "\$") {  # q($) confuses emacs
-			    # dblog("not a scalar ($h1)\n");
-			    # Get the main evaluator to eval this
-			    push @results, ["$h", undef, 1];
-			} elsif (exists $packageName->{$h2}) {
-			    # dblog("'$h' lives in package '$packageName'\n") if $ldebug;
-			    push @results, ["$h", $tmp, 0];
-			} else {
-			    # dblog("'$h' is in no package\n") if $ldebug;
-			    push @results, ["$h", undef, 1];
-			}
-		    }
-		}
-		select $oh;
-  	        if ($ldebug) {
-		    dblog("vars:", DB::Data::Dump::dump(@results), "\n");
-	        }
-	    } else {
-		dblog("getContextProperties -- don't have dumpValue");
-	    }
-	};
-	if ($@) {
-	    dblog($@);
-	    die "code:(1):error:($@)";
+	for my $key (keys %$stash) {
+	    next if $key =~ /^(?:_<|[^0a-zA-Z_])/;
+	    next if $key =~ /::$/;
+	    my $glob = \($stash->{$key});
+	    my $has_scalar = $] >= 5.010 ?
+		!B::svref_2object($glob)->SV->isa('B::SPECIAL') :
+		defined ${*{$glob}{SCALAR}};
+	    my $array = *{$glob}{ARRAY};
+	    my $hash = *{$glob}{HASH};
+	    next unless $has_scalar || $array || $hash;
+	    push @results, ["\$${key}", undef, 1] if $has_scalar;
+	    push @results, ["\@${key}", undef, 1] if $array;
+	    push @results, ["\%${key}", undef, 1] if $hash;
 	}
 	return \@results;
-    } elsif ($context_id == FunctionArguments) {
-        # This should be evaluated in the caller, in the main event
-	# loop in DB::DB
-	@results = (['@_', undef, 1]);
-	return \@results;
     } elsif ($context_id == PunctuationVariables) {
 	my ($packageName, $filename, $line) = caller(1);
 	my %vals;
@@ -270,6 +235,10 @@ sub getContextProperties($$) {
 	foreach my $pv (@_punctuationVariables) {
 	    push (@results, [$pv, undef, 1]);
 	}
+	foreach my $pv (@_rxPunctuationVariables) {
+	    # somebody might use @' and trigger the condition, I'm willing to risk it
+	    push (@results, ["\$$pv", undef, exists $main::{$pv} ? 1 : 0]);
+	}
 	return \@results;
 	
     } else {
@@ -303,61 +272,6 @@ sub _truncateIfNecessary {
     return $res;
 }
 
-sub emitFinalPropertyValue($$$$$) {
-    my ($cmd,
-	$transactionID,
-	$property_long_name,
-	$valRefs,
-	$maxDataSize) = @_;
-    my $propValue;
-    my $res;
-    if (scalar @$valRefs > 1 || $property_long_name =~ /^[\@\%][_:\w]+$/) {
-	# Call in array context to get value back
-	$DB::Data::Dump::maxDataSize = $maxDataSize;
-	local $@;
-	eval {
-	    ($res) = DB::Data::Dump::dump(@$valRefs);
-	    $res = _truncateIfNecessary($res, $maxDataSize, 1);
-	    $propValue = "($res)";
-	};
-	if ($@) {
-	    dblog("Error in emitFinalPropertyValue: [[$@]]");
-	    $propValue = "";
-	}
-    } else {
-	my $val = $valRefs->[0];
-	if (ref $val) {
-	    $DB::Data::Dump::maxDataSize = $maxDataSize;
-	    ($res) = DB::Data::Dump::dump($val);
-	    $res = _truncateIfNecessary($res, $maxDataSize, 1);
-	    if (ref($val) =~ /ARRAY/) {
-		$propValue = "[$res]";
-	    } elsif (ref($val) =~ /HASH/) {
-		$propValue = "{$res}";
-	    } else {
-		$propValue = "($res)";
-	    }
-	} else {
-	    $propValue = _truncateIfNecessary($val, $maxDataSize, 0);
-	}
-    }
-    my $size = length $propValue;
-    ($encoding, $encVal) = figureEncoding($propValue);
-    $res = sprintf(qq(%s\n<response %s command="%s" 
-		      transaction_id="%s"
-		      size="%d"
-		      encoding="%s">%s</response>),
-		   xmlHeader(),
-		   namespaceAttr(),
-		   $cmd,
-		   $transactionID,
-		   $size,
-		   $encoding,
-		   $encVal);
-    printWithLength($res);
-}
-
-
 sub emitEvalResultAsProperty($$$$$) {
     my ($cmd,
 	$transactionID,
@@ -381,44 +295,6 @@ sub emitEvalResultAsProperty($$$$$) {
     printWithLength($res);
 }
 
-# This routine either returns a (name, value, evalRequest) array
-# Locals are evaluated here everywhere but at the top-level
-# Everthing else we attempt to evaluate here.  If we fail, we'll
-# get the DB::DB main loop to eval it, and then the second
-# function will fill in the full value.
-
-# Precondition: we're inside an eval block.
-
-sub getPropertyValue($$$$$$$) {
-    my ($cmd,
-	$context_id,
-	$requestedStackDepth,
-	$currStackSize,
-	$packageName,
-	$property_long_name,
-	$callDepthAdjustment) = @_;
-
-    my $finalVal;
-    if ($context_id == FunctionArguments) {
-	# Get the args, and then get the full property
-        my @xargs;
-	{
-	    package DB;
-	    my ($packageName, $filename, $line) = caller($requestedStackDepth + $callDepthAdjustment - 1);
-	    dblog("getPropertyValue: curr args are [", join(", ", @DB::args), "]");
-	    @xargs = @DB::args;
-	}
-	if (@xargs) {
-	    my $indexer = $property_long_name;
-	    if ($indexer =~ m/^\$_\[(\d+)\]/) {
-		return [$property_long_name, $xargs[$indexer], 0];
-	    }
-	}
-    }
-    return [$property_long_name, undef, 1];
-}
-
-
 sub getPropertyInfo($$) {
     my ($property_long_name, $propertyKey) = @_;
 
@@ -471,7 +347,13 @@ sub _attr_needs_base64_encoding {
 }
 
 sub getFullPropertyInfoByValue($$$$$$) {
-    my ($name,
+    _getFullPropertyInfoByValue('<property', '</property>', @_);
+}
+
+sub _getFullPropertyInfoByValue {
+    my ($startTag,
+	$endTag,
+	$name,
 	$fullname,
 	$val,
 	$maxDataSize,
@@ -480,7 +362,7 @@ sub getFullPropertyInfoByValue($$$$$$) {
 	) = @_;
     # dblog("getFullPropertyInfoByValue: (@_)\n");
     my $encoding;
-    my $res = '<property';
+    my $res = $startTag;
     if ($currentDepth > 0) {
 	$res .= propertyTagSpacer($currentDepth);
     }
@@ -663,7 +545,7 @@ sub getFullPropertyInfoByValue($$$$$$) {
 	    $res .= qq( >);
 	    $res .= _getFullPropertyInfoByValue_emitNames($maxDataSize, %b_attr);
 	}
-	$res .= qq(</property>\n);
+	$res .= $endTag . qq(\n);
     } else {
 	$res .= qq( size="$encValLength") if defined $encValLength;
 	$res .= qq( >);
@@ -677,7 +559,7 @@ sub getFullPropertyInfoByValue($$$$$$) {
 			    $encoding ? qq( encoding="$encoding") : "",
 			    $encVal);
 	}
-	$res .= "</property>";
+	$res .= $endTag;
 	$res .= "\n" if $currentDepth == 0;
     }
     # dblog("getFullPropertyInfoByValue: \{$res}\n");
